@@ -12,7 +12,7 @@ use Sys::Syslog qw(:macros);
 use LISM::Storage;
 use Data::Dumper;
 
-our $VERSION = '2.1.5';
+our $VERSION = '2.1.6';
 
 our $lism_master = 'lism_master';
 our $syncrdn = 'cn=sync';
@@ -433,7 +433,11 @@ sub add
     my $rc;
 
     $entryStr =~ s/\n\s*([^:]+)\n/$1\n/g;
-    my ($dn) = ($entryStr =~ /^dn: (.*)$/m);
+    my ($dn) = ($entryStr =~ /^dn:{1,2} (.*)$/m);
+    if ($entryStr =~ /^dn::/) {
+        $dn = decode_base64($dn);
+    }
+
     $entryStr =~ s/^dn:.*\n//;
     $dn = lc $dn ;
     $dn =~ s/,\s+/,/g;
