@@ -82,6 +82,19 @@ sub init
 
 =pod
 
+=head2 getOrder
+
+Get order to do handler.
+
+=cut
+
+sub getOrder
+{
+    return 'middle';
+}
+
+=pod
+
 =head2 pre_bind
 
 This method is called bofore L<LISM> do the bind operation.
