@@ -187,6 +187,10 @@ sub search
         return (LDAP_FILTER_ERROR, ());
     }
 
+    if (!$sizeLim) {
+        $sizeLim = -1;
+    }
+
     if ($self->_getConnect()) {
         return (LDAP_SERVER_DOWN, ());
     }
@@ -338,6 +342,7 @@ sub add
 
     # check rdn's value
     my ($rdn, $rdn_val) = ($dn =~ /^([^=]+)=([^,]+),/);
+    $rdn_val =~ s/([.*+?\[\]()])/\\$1/g;
     if ($entryStr !~ /^$rdn: $rdn_val$/mi) {
         return LDAP_NAMING_VIOLATION;
     }
@@ -707,6 +712,14 @@ sub _checkConfig
                 $self->log(level => 'alert', message => "rdn of $oname object is invalid");
                 return 1;
             }
+
+            foreach my $si (@{$oconf->{strginfo}}) {
+                # check type of value
+                if (defined($si->{value}) && !ref($si->{value}[0])) {
+                    $self->log(level => 'alert', message => "type of strginfo value doesn't exist");
+                    return 1;
+                }
+            }
         }
 
         # check container's link
