@@ -156,7 +156,6 @@ sub _objSearch
             chop;
             my @data = split(/$dlm/);
             my $entry;
-            my $rdn_val = $data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]];
 
             # check the number of returned entries
             if ($sizeLim >= 0 && @match_entries == $sizeLim) {
@@ -174,14 +173,13 @@ sub _objSearch
             }
 
             # get all values of the entry
-            my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             foreach my $oc (@{$oconf->{oc}}) {
                 $entry = $entry."objectclass: $oc\n";
             }
             foreach my $attr (keys %{$oconf->{attr}}) {
                 if (defined($oconf->{attr}{$attr}->{column})) {
                     foreach my $val (split(/$valdlm/, $data[$oconf->{attr}{$attr}->{column}[0]])) {
-
+                        $val =~ s/ *$//;
                         $entry = $entry."$attr: $val\n";
                     }
                 } else {
@@ -197,9 +195,10 @@ sub _objSearch
                 }
             }
 
-            $rdn = decode($conf->{mbcode}[0], $rdn);
             $entry = decode($conf->{mbcode}[0], $entry);
 
+            my ($rdn_val) = ($entry =~ /^$oconf->{rdn}[0]: (.*)$/mi);
+            my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             $entry = "dn: $rdn,$suffix\n$entry";
 
             # parse filter
@@ -235,9 +234,10 @@ sub _objModify
     my $rc = LDAP_SUCCESS;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
+    $rdn_val =~ s/([.*+?\[\]])/\\$1/g;
 
     # multibyte string
-    $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
+    $rdn_val = encode($conf->{mbcode}[0], $rdn_val);
 
     my $lock;
     my $file;
@@ -336,8 +336,10 @@ sub _objModify
 
                 if($action eq "ADD") {
                     # check whether the value already exists
-                    foreach my $val (@values) {
-                        if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$val *$valdlm/i) {
+                    for (my $i = 0; $i < @values; $i++) {
+                        my $value = $values[0];
+                        $value =~ s/([.*+?\[\]])/\\$1/g;
+                        if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$value *$valdlm/i) {
                             $rc = LDAP_TYPE_OR_VALUE_EXISTS;
                             last DO;
                         }
@@ -350,10 +352,12 @@ sub _objModify
                 } elsif($action eq "DELETE") {
                     if (@values && $values[0]) {
                         # check whether the value exists
-                        foreach my $val (@values) {
-                            if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$val *$valdlm/i) {
+                        for (my $i = 0;  $i < @values; $i++) {
+                            my $value = $values[0];
+                            $value =~ s/([.*+?\[\]])/\\$1/g;
+                            if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$value *$valdlm/i) {
                                 my $str = "$valdlm$data[$coln]$valdlm";
-                                $str =~ s/$valdlm$val$valdlm/$valdlm/i;
+                                $str =~ s/$valdlm$value$valdlm/$valdlm/i;
                                 ($data[$coln]) = ($str =~ /^$valdlm(.*)$valdlm$/);
                             } else {
                                 $rc = LDAP_NO_SUCH_ATTRIBUTE;
@@ -405,6 +409,7 @@ sub _objAdd
     my $rc = LDAP_SUCCESS;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
+    $rdn_val =~ s/([.*+?\[\]])/\\$1/g;
 
     # multibyte string
     $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
@@ -538,6 +543,7 @@ sub _objDelete
     my $rc = LDAP_NO_SUCH_OBJECT;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
+    $rdn_val =~ s/([.*+?\[\]])/\\$1/g;
 
     # multibyte string
     $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
@@ -663,7 +669,9 @@ sub _getParentRdn
 
         # check the data corresponds to the object's id
         if ($data[$poconf->{id}[0]->{column}[0]] =~ /^$pkey$/i) {
-            $prdn = "$poconf->{rdn}[0]=$data[$poconf->{attr}{$poconf->{rdn}[0]}->{column}[0]]";
+            my $rdn_val = $data[$poconf->{attr}{$poconf->{rdn}[0]}->{column}[0]];
+            $rdn_val =~ s/ *$//;
+            $prdn = "$poconf->{rdn}[0]=$rdn_val";
             last;
         }
     }
@@ -718,7 +726,9 @@ sub _getAttrValues
         for (my $i = 0; $i < @keys; $i++) {
             if (defined($aoconf)) {
                 if ($data[$aoconf->{id}[0]->{column}[0]] =~ /^$keys[$i]$/i) {
-                    $attrStr = $attrStr."$attr: $aoconf->{rdn}[0]=$data[$aoconf->{attr}{$aoconf->{rdn}[0]}->{column}[0]],".$self->_getParentDn($aobj, $data[$aoconf->{id}[0]->{column}[0]])."\n";
+                    my $rdn_val = $data[$aoconf->{attr}{$aoconf->{rdn}[0]}->{column}[0]];
+                    $rdn_val =~ s/ *$//;
+                    $attrStr = $attrStr."$attr: $aoconf->{rdn}[0]=$rdn_val,".$self->_getParentDn($aobj, $data[$aoconf->{id}[0]->{column}[0]])."\n";
                     splice(@keys, $i, 1);
                     last;
                 }
@@ -728,6 +738,7 @@ sub _getAttrValues
                     foreach my $coln (@colnums) {
                         $value =~ s/%$coln/$data[$coln]/g;
                     }
+                    $value =~ s/ *$//;
 
                     if ($value) {
                         $attrStr = $attrStr."$attr: $value\n";
@@ -814,6 +825,7 @@ sub _getAttrKeys
             my @data = split(/$dlm/);
             my $dvals;
             foreach my $coln (@colnums) {
+                $data[$coln] =~ s/ *$//;
                 if ($dvals) {
                     $dvals = "$dvals;$data[$coln]";
                 } else {
