@@ -143,12 +143,15 @@ sub search
         } else {
             $filterStr =~ s/\($ldapmap->{local}=/\($ldapmap->{foreign}=/mi;
         }
+
+        if (Encode::is_utf8($filterStr)) {
+            Encode::_utf8_off($filterStr);
+        }
     }
 
     $base =~ s/$self->{suffix}$/$conf->{nc}/i;
     $base = decode('utf8', $base);
     $filterStr =~ s/$self->{suffix}(\)*)/$conf->{nc}$1/gi;
-    $filterStr = encode('utf8', $filterStr);
     my $msg = $self->{ldap}->search(base => $base, scope => $scope, deref => $deref, sizeLimit => $sizeLim, timeLimit => $timeLim, filter => $filterStr);
 
     if (!$msg->code) {
