@@ -131,7 +131,13 @@ sub _getConnect
     my $conf = $self->{_config};
 
     if (defined($self->{db})) {
-        return 0;
+        my $sth = $self->{db}->prepare("select 'check';");
+        if (!$sth->execute) {
+            $self->log(level => 'error', message => "Connection check failed");
+        } else {
+            return 0;
+        }
+        $sth->finish;
     }
 
     $self->{db} = DBI->connect($conf->{dsn}[0], $conf->{admin}[0], $conf->{passwd}[0]);
@@ -223,7 +229,7 @@ sub _objSearch
 
     DO: {
         # get data of the entries
-        my $sql = "select distinct $oconf->{table}[0].$oconf->{id}[0]->{column}[0], $oconf->{table}[0].$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]";
+        my $sql = "select distinct $oconf->{table}[0].$oconf->{id}[0]->{column}[0]";
         foreach my $attr ('objectclass', keys %{$oconf->{attr}}) {
             if (!defined($oconf->{attr}{$attr}) || !defined($oconf->{attr}{$attr}->{column})) {
                 next;
@@ -282,8 +288,6 @@ sub _objSearch
                 next;
             }
 
-            my $rdn = "$oconf->{rdn}[0]=".shift(@data);
-
             my $j = 0;
             foreach my $attr ('objectclass', keys %{$oconf->{attr}}) {
                 if ($attr =~ /^objectclass$/i) {
@@ -293,9 +297,9 @@ sub _objSearch
                 } elsif (!defined($oconf->{attr}->{$attr})) {
                     next;
                 } elsif (defined($oconf->{attr}{$attr}->{column})) {
-                    if ($data[$j]) {
-                        my $value = $data[$j];
-
+                    my $value = $data[$j];
+                    $value =~ s/ *$//;
+                    if ($value) {
                         $entry = $entry."$attr: $value\n";
                     }
                     $j++;
@@ -317,19 +321,16 @@ sub _objSearch
             # multibyte value
             if ($conf->{dsn}[0] =~ /^DBI:Oracle/i && Encode::is_utf8($entry)) {
                 # add-hoc method for Oracle
-                if (Encode::is_utf8($rdn)) {
-                    Encode::_utf8_off($rdn);
-                }
                 if (Encode::is_utf8($entry)) {
                     Encode::_utf8_off($entry);
                 }
             }
 
-            Encode::from_to($rdn, $conf->{mbcode}[0], 'utf8');
-            $rdn = decode('utf8', $rdn);
             Encode::from_to($entry, $conf->{mbcode}[0], 'utf8');
             $entry = decode('utf8', $entry);
 
+            my ($rdn_val) = ($entry =~ /^$oconf->{rdn}[0]: (.*)$/mi);
+            my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             $entry = "dn: $rdn,$suffix\n$entry";
             push(@match_entries, $self->_pwdFormat($entry));
             push(@match_keys, $key);
@@ -383,7 +384,6 @@ sub _objModify
             my $action = shift @list;
             my $attr    = lc(shift @list);
             my @values;
-            my $value;
             my $sql;
             my $sth;
 
@@ -397,9 +397,16 @@ sub _objModify
 
             # can't modify the attribute for rdn
             if ($attr eq $oconf->{rdn}[0]) {
-                if ($action ne "REPLACE" || @values != 1 || !($entry =~ /^$attr: $values[0]$/mi)) {
+                if ($action ne "REPLACE" || @values != 1) {
                     $rc = LDAP_CONSTRAINT_VIOLATION;
                     last DO;
+                } else {
+                    my $value = $values[0];
+                    $value =~ s/([.*+?\[\]])/\\$1/g;
+                    if ($entry !~ /^$attr: $value$/mi) {
+                        $rc = LDAP_CONSTRAINT_VIOLATION;
+                        last DO;
+                    }
                 }
             }
 
@@ -414,6 +421,7 @@ sub _objModify
             if($action eq "ADD") {
                 # check whether the value already exists
                 my $vals_str = "(".join('|', @values).")";
+                $vals_str =~ s/([.*+?\[\]])/\\$1/g;
                 if ($entry =~ /^$attr: $vals_str *$/mi) {
                     $rc = LDAP_TYPE_OR_VALUE_EXISTS;
                     last DO;
@@ -440,8 +448,8 @@ sub _objModify
                 }
 
                 if (defined($oconf->{attr}{$attr}->{addproc})) {
-                    foreach $value (@values) {
-                        if ($self->_addAttrValues($oconf, $key, $attr, $value)) {
+                    for (my $i = 0; $i < @values; $i++) {
+                        if ($self->_addAttrValues($oconf, $key, $attr, $values[$i])) {
                             $rc = LDAP_OPERATIONS_ERROR;
                             last DO;
                         }
@@ -449,18 +457,16 @@ sub _objModify
                 }
             } elsif( $action eq "DELETE" ) {
                 # check whether the value exists
-                foreach $value (@values) {
-                    if ($value && !($entry =~ /^$attr: $value *$/mi)) {
+                for (my $i = 0; $i < @values; $i++) {
+                    my $value = $values[$i];
+                    $value =~ s/([.*+?\[\]])/\\$1/g;
+                    if ($value && $entry !~ /^$attr: $value *$/mi) {
                         $rc = LDAP_NO_SUCH_ATTRIBUTE;
                         last DO;
                     }
                 }
 
                 if (defined($oconf->{attr}{$attr}->{column})) {
-                    if (@values && (@values > 1 || $values[0] && !($entry =~ /^$attr: $values[0] *$/mi))) {
-                        $rc = LDAP_NO_SUCH_ATTRIBUTE;
-                        last DO;
-                    }
                     $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
 
                     # multibyte value
@@ -479,9 +485,14 @@ sub _objModify
                 if (defined($oconf->{attr}{$attr}->{delproc})) {
                     if (!@values || !$values[0]) {
                         @values = ($entry =~ /^$attr: (.*)$/mgi);
+                        for (my $i = 0; $i < @values; $i++) {
+                            # escape special character
+                            $values[$i] =~ s/'/''/g;
+                            $values[$i] =~ s/\\/\\\\/g;
+                        }
                     }
 
-                    foreach $value (@values) {
+                    foreach my $value (@values) {
                         if ($self->_delAttrValues($oconf, $key, $attr, $value)) {
                             $rc = LDAP_OPERATIONS_ERROR;
                             last DO;
@@ -516,7 +527,9 @@ sub _objModify
                     # delete the old values which exists
                     foreach my $old_val (@old_vals) {
                         my $match = 0;
-                        foreach $value (@values) {
+                        for (my $i = 0; $i < @values; $i++) {
+                            my $value = $values[$i];
+                            $value =~ s/([.*+?\[\]])/\\$1/g;
                             if ($old_val =~ /^$value$/i) {
                                 $match = 1;
                                 next;
@@ -526,6 +539,11 @@ sub _objModify
                             push(@existing_vals, $old_val);
                             next;
                         }
+
+                        # escape special character
+                        $old_val =~ s/'/''/g;
+                        $old_val =~ s/\\/\\\\/g;
+
                         if ($self->_delAttrValues($oconf, $key, $attr, $old_val)) {
                             $rc = LDAP_OPERATIONS_ERROR;
                             last DO;
@@ -533,11 +551,14 @@ sub _objModify
                     }
 
                     # add the new values doesn't exist
-                    foreach $value (@values) {
+                    for (my $i = 0; $i < @values; $i++) {
+                        my $value = $values[$i];
+                        $value =~ s/([.*+?\[\]])/\\$1/g;
                         if (grep(/^$value$/i, @existing_vals)) {
                             next;
                         }
-                        if ($self->_addAttrValues($oconf, $key, $attr, $value)) {
+
+                        if ($self->_addAttrValues($oconf, $key, $attr, $values[$i])) {
                             $rc = LDAP_OPERATIONS_ERROR;
                             last DO;
                         }
@@ -874,7 +895,10 @@ sub _getParentRdn
     my @data = $sth->fetchrow_array;
     $sth->finish;
 
-    return ("$poconf->{rdn}[0]=$data[1]", $data[0]);
+    my $rdn_val = $data[1];
+    $rdn_val =~ s/ *$//;
+
+    return ("$poconf->{rdn}[0]=$rdn_val", $data[0]);
 }
 
 sub _getAttrValues
@@ -895,12 +919,27 @@ sub _getAttrValues
     if (defined($oconf->{attr}{$attr}->{oname})) {
         $aobj = $self->{object}{$oconf->{attr}{$attr}->{oname}[0]};
         $aoconf = $aobj->{conf};
-        $selexpr = "$aoconf->{table}[0].$aoconf->{id}[0]->{column}[0], $aoconf->{table}[0].$aoconf->{attr}{$aoconf->{rdn}[0]}->{column}[0]";
-        $from = "$from, $aoconf->{table}[0]";
+        $selexpr = "$aoconf->{table}[0].$aoconf->{id}[0]->{column}[0]";
+        if (defined($aoconf->{attr}{$aoconf->{rdn}[0]}->{selexpr})) {
+            $selexpr = "$selexpr, $aoconf->{attr}{$aoconf->{rdn}[0]}->{selexpr}[0]";
+        } else {
+            $selexpr = "$selexpr, $aoconf->{table}[0].$aoconf->{attr}{$aoconf->{rdn}[0]}->{column}[0]";
+        }
+
+        if ($oconf->{table}[0] ne $aoconf->{table}[0]) {
+            $from = "$from, $aoconf->{table}[0]";
+        }
+
         if (defined($oconf->{attr}{$attr}->{cname})) {
             $cobj = $self->{object}{$oconf->{attr}{$attr}->{cname}[0]};
             $coconf = $cobj->{conf};
-            $selexpr = "$selexpr, $coconf->{table}[0].$coconf->{id}[0]->{column}[0], $coconf->{table}[0].$coconf->{attr}{$coconf->{rdn}[0]}->{column}[0]";
+            $selexpr = "$selexpr, $coconf->{table}[0].$coconf->{id}[0]->{column}[0]";
+            if (defined($aoconf->{attr}{$aoconf->{rdn}[0]}->{selexpr})) {
+                $selexpr = "$selexpr, $coconf->{attr}{$coconf->{rdn}[0]}->{selexpr}[0]";
+            } else {
+                $selexpr = "$selexpr, $coconf->{table}[0].$coconf->{attr}{$coconf->{rdn}[0]}->{column}[0]";
+            }
+
             $from = "$from, $coconf->{table}[0]";
         }
     } else {
@@ -934,7 +973,7 @@ sub _getAttrValues
             } else {
                 $attrStr = $attrStr.$self->_getParentDn($aobj, $data[0])."\n";
             }
-            $attrStr =~ s/ *,/,/;
+            $attrStr =~ s/ *,/,/g;
         } else{
             $data[0] =~ s/ *$//;
             if ($data[0]) {
@@ -1097,7 +1136,7 @@ sub _filter2sql
 
     if (!$filter) {
         ${$from} = $oconf->{table}[0];
-        ${$where} = 1;
+        ${$where} = '1 = 1';
         return 1;
     }
 
@@ -1174,6 +1213,7 @@ sub _filter2sql
         ${$where} = "lower($oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0]) $type lower(\'$value\')";
     } elsif (defined($oconf->{attr}{$attr}->{oname})) {
         my $attrobj = $conf->{object}{$oconf->{attr}{$attr}->{oname}[0]};
+        my $aconf = $attrobj->{conf};
         if (defined($oconf->{attr}{$attr}->{fromtbls})) {
             ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
         }
@@ -1183,6 +1223,19 @@ sub _filter2sql
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             ${$where} = "${$where} and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
+
+        if (defined($aconf->{container}) && !defined($aconf->{container}[0]->{rdn})) {
+            my $paconf = $self->{object}{$aconf->{container}[0]->{oname}[0]}->{conf};
+
+            $from = "$from, $paconf->{table}[0]";
+            if (defined($aconf->{container}[0]->{fromtbls})) {
+                $from = "$from, $oconf->{container}[0]->{fromtbls}[0]";
+            }
+
+            if (defined($aconf->{container}[0]->{joinwhere})) {
+                $where = "$where and $aconf->{container}[0]->{joinwhere}[0]";
+            }
+        }
     } else {
         if (defined($oconf->{attr}{$attr}->{fromtbls})) {
             ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
